@@ -1,4 +1,6 @@
 import asyncio
+from botstory.ast import story_context
+from botstory.utils import answer
 import logging
 from unittest import mock
 import pytest
@@ -6,7 +8,7 @@ import pytest
 from . import messenger
 from .. import commonhttp, mockdb, mockhttp
 from ... import di, Story, utils
-from ...middlewares import any, option
+from ...middlewares import any, option, sticker
 
 logger = logging.getLogger(__name__)
 
@@ -31,7 +33,7 @@ async def test_send_text_message():
     await story.start()
 
     await interface.send_text_message(
-        recipient=user, text='hi!', options=None
+        recipient=user, text='hi!', quick_replies=None
     )
 
     mock_http.post.assert_called_with(
@@ -50,6 +52,273 @@ async def test_send_text_message():
     )
 
 
+@pytest.mark.asyncio
+async def test_truncate_long_message():
+    user = utils.build_fake_user()
+
+    global story
+    story = Story()
+
+    interface = story.use(messenger.FBInterface(page_access_token='qwerty1'))
+    mock_http = story.use(mockhttp.MockHttpInterface())
+
+    await story.start()
+
+    very_long_message = 'very_long_message' * 100
+    await interface.send_text_message(
+        recipient=user,
+        text=very_long_message,
+        quick_replies=None,
+        options={
+            'overflow': 'cut'
+        }
+    )
+
+    mock_http.post.assert_called_with(
+        'https://graph.facebook.com/v2.6/me/messages/',
+        params={
+            'access_token': 'qwerty1',
+        },
+        json={
+            'message': {
+                'text': very_long_message[:640],
+            },
+            'recipient': {
+                'id': user['facebook_user_id'],
+            },
+        }
+    )
+
+
+@pytest.mark.asyncio
+async def test_truncate_with_ellipsis_long_message_by_default():
+    user = utils.build_fake_user()
+
+    global story
+    story = Story()
+
+    interface = story.use(messenger.FBInterface(page_access_token='qwerty1'))
+    mock_http = story.use(mockhttp.MockHttpInterface())
+
+    await story.start()
+
+    very_long_message = 'very_long_message' * 100
+    await interface.send_text_message(
+        recipient=user,
+        text=very_long_message,
+        quick_replies=None,
+    )
+
+    mock_http.post.assert_called_with(
+        'https://graph.facebook.com/v2.6/me/messages/',
+        params={
+            'access_token': 'qwerty1',
+        },
+        json={
+            'message': {
+                'text': very_long_message[:638] + '\u2026',
+            },
+            'recipient': {
+                'id': user['facebook_user_id'],
+            },
+        }
+    )
+
+
+@pytest.mark.asyncio
+async def test_send_list():
+    with answer.Talk() as talk:
+        story = talk.story
+        fb_interface = story.use(messenger.FBInterface(page_access_token='qwerty1'))
+        mock_http = story.use(mockhttp.MockHttpInterface())
+        await story.start()
+        await fb_interface.send_list(
+            recipient=talk.user,
+            elements=[{
+                'title': 'Classic T-Shirt Collection',  # (*) required
+                'image_url': 'https://peterssendreceiveapp.ngrok.io/img/collection.png',
+                'subtitle': 'See all our colors',
+                'default_action': {
+                    'type': 'web_url',
+                    'url': 'https://peterssendreceiveapp.ngrok.io/shop_collection',
+                    'messenger_extensions': True,
+                    'webview_height_ratio': 'tall',
+                    'fallback_url': 'https://peterssendreceiveapp.ngrok.io/'
+                },
+                'buttons': [{
+                    'title': 'View',
+                    'type': 'web_url',
+                    'url': 'https://peterssendreceiveapp.ngrok.io/collection',
+                    'messenger_extensions': True,
+                    'webview_height_ratio': 'tall',
+                    'fallback_url': 'https://peterssendreceiveapp.ngrok.io/'
+                }]
+            }, {
+                'title': 'Classic White T-Shirt',
+                'image_url': 'https://peterssendreceiveapp.ngrok.io/img/white-t-shirt.png',
+                'subtitle': '100% Cotton, 200% Comfortable',
+                'default_action': {
+                    'type': 'web_url',
+                    'url': 'https://peterssendreceiveapp.ngrok.io/view?item=100',
+                    'messenger_extensions': True,
+                    'webview_height_ratio': 'tall',
+                    'fallback_url': 'https://peterssendreceiveapp.ngrok.io/'
+                },
+                'buttons': [{
+                    'title': 'Shop Now',
+                    'type': 'web_url',
+                    'url': 'https://peterssendreceiveapp.ngrok.io/shop?item=100',
+                    'messenger_extensions': True,
+                    'webview_height_ratio': 'tall',
+                    'fallback_url': 'https://peterssendreceiveapp.ngrok.io/'
+                }]
+            }, {
+                'title': 'Classic Blue T-Shirt',
+                'image_url': 'https://peterssendreceiveapp.ngrok.io/img/blue-t-shirt.png',
+                'subtitle': '100% Cotton, 200% Comfortable',
+                'default_action': {
+                    'type': 'web_url',
+                    'url': 'https://peterssendreceiveapp.ngrok.io/view?item=101',
+                    'messenger_extensions': True,
+                    'webview_height_ratio': 'tall',
+                    'fallback_url': 'https://peterssendreceiveapp.ngrok.io/'
+                },
+                'buttons': [{
+                    'title': 'Shop Now',
+                    'type': 'web_url',
+                    'url': 'https://peterssendreceiveapp.ngrok.io/shop?item=101',
+                    'messenger_extensions': True,
+                    'webview_height_ratio': 'tall',
+                    'fallback_url': 'https://peterssendreceiveapp.ngrok.io/'
+                }]
+            }, {
+                'title': 'Classic Black T-Shirt',
+                'image_url': 'https://peterssendreceiveapp.ngrok.io/img/black-t-shirt.png',
+                'subtitle': '100% Cotton, 200% Comfortable',
+                'default_action': {
+                    'type': 'web_url',
+                    'url': 'https://peterssendreceiveapp.ngrok.io/view?item=102',
+                    'messenger_extensions': True,
+                    'webview_height_ratio': 'tall',
+                    'fallback_url': 'https://peterssendreceiveapp.ngrok.io/'
+                },
+                'buttons': [{
+                    'title': 'Shop Now',
+                    'type': 'web_url',
+                    'url': 'https://peterssendreceiveapp.ngrok.io/shop?item=102',
+                    'messenger_extensions': True,
+                    'webview_height_ratio': 'tall',
+                    'fallback_url': 'https://peterssendreceiveapp.ngrok.io/'
+                }]
+            }], buttons=[{
+                'title': 'View More',
+                'payload': 'payload',
+            }])
+        mock_http.post.assert_called_with(
+            'https://graph.facebook.com/v2.6/me/messages/',
+            params={
+                'access_token': 'qwerty1',
+            },
+            json={
+                'message': {
+                    'attachment': {
+                        'type': 'template',
+                        'payload': {
+                            'template_type': 'list',
+                            'elements': [{
+                                'title': 'Classic T-Shirt Collection',  # (*) required
+                                'image_url': 'https://peterssendreceiveapp.ngrok.io/img/collection.png',
+                                'subtitle': 'See all our colors',
+                                'default_action': {
+                                    'type': 'web_url',
+                                    'url': 'https://peterssendreceiveapp.ngrok.io/shop_collection',
+                                    'messenger_extensions': True,
+                                    'webview_height_ratio': 'tall',
+                                    'fallback_url': 'https://peterssendreceiveapp.ngrok.io/'
+                                },
+                                'buttons': [{
+                                    'title': 'View',
+                                    'type': 'web_url',
+                                    'url': 'https://peterssendreceiveapp.ngrok.io/collection',
+                                    'messenger_extensions': True,
+                                    'webview_height_ratio': 'tall',
+                                    'fallback_url': 'https://peterssendreceiveapp.ngrok.io/'
+                                }]
+                            }, {
+                                'title': 'Classic White T-Shirt',
+                                'image_url': 'https://peterssendreceiveapp.ngrok.io/img/white-t-shirt.png',
+                                'subtitle': '100% Cotton, 200% Comfortable',
+                                'default_action': {
+                                    'type': 'web_url',
+                                    'url': 'https://peterssendreceiveapp.ngrok.io/view?item=100',
+                                    'messenger_extensions': True,
+                                    'webview_height_ratio': 'tall',
+                                    'fallback_url': 'https://peterssendreceiveapp.ngrok.io/'
+                                },
+                                'buttons': [{
+                                    'title': 'Shop Now',
+                                    'type': 'web_url',
+                                    'url': 'https://peterssendreceiveapp.ngrok.io/shop?item=100',
+                                    'messenger_extensions': True,
+                                    'webview_height_ratio': 'tall',
+                                    'fallback_url': 'https://peterssendreceiveapp.ngrok.io/'
+                                }]
+                            }, {
+                                'title': 'Classic Blue T-Shirt',
+                                'image_url': 'https://peterssendreceiveapp.ngrok.io/img/blue-t-shirt.png',
+                                'subtitle': '100% Cotton, 200% Comfortable',
+                                'default_action': {
+                                    'type': 'web_url',
+                                    'url': 'https://peterssendreceiveapp.ngrok.io/view?item=101',
+                                    'messenger_extensions': True,
+                                    'webview_height_ratio': 'tall',
+                                    'fallback_url': 'https://peterssendreceiveapp.ngrok.io/'
+                                },
+                                'buttons': [{
+                                    'title': 'Shop Now',
+                                    'type': 'web_url',
+                                    'url': 'https://peterssendreceiveapp.ngrok.io/shop?item=101',
+                                    'messenger_extensions': True,
+                                    'webview_height_ratio': 'tall',
+                                    'fallback_url': 'https://peterssendreceiveapp.ngrok.io/'
+                                }]
+                            }, {
+                                'title': 'Classic Black T-Shirt',
+                                'image_url': 'https://peterssendreceiveapp.ngrok.io/img/black-t-shirt.png',
+                                'subtitle': '100% Cotton, 200% Comfortable',
+                                'default_action': {
+                                    'type': 'web_url',
+                                    'url': 'https://peterssendreceiveapp.ngrok.io/view?item=102',
+                                    'messenger_extensions': True,
+                                    'webview_height_ratio': 'tall',
+                                    'fallback_url': 'https://peterssendreceiveapp.ngrok.io/'
+                                },
+                                'buttons': [{
+                                    'title': 'Shop Now',
+                                    'type': 'web_url',
+                                    'url': 'https://peterssendreceiveapp.ngrok.io/shop?item=102',
+                                    'messenger_extensions': True,
+                                    'webview_height_ratio': 'tall',
+                                    'fallback_url': 'https://peterssendreceiveapp.ngrok.io/'
+                                }]
+                            }],
+                            'buttons': [
+                                {
+                                    'title': 'View More',
+                                    'type': 'postback',
+                                    'payload': 'payload'
+                                }
+                            ]
+                        }
+                    }
+                },
+                'recipient': {
+                    'id': talk.user['facebook_user_id'],
+                },
+            }
+        )
+
+
 @pytest.mark.asyncio
 async def test_integration():
     user = utils.build_fake_user()
@@ -80,7 +349,7 @@ async def test_integration():
 
 
 @pytest.mark.asyncio
-async def test_options():
+async def test_quick_replies():
     user = utils.build_fake_user()
 
     global story
@@ -92,7 +361,7 @@ async def test_options():
 
     await story.ask(
         'Which color do you like?',
-        options=[{
+        quick_replies=[{
             'title': 'Red',
             'payload': 0xff0000,
         }, {
@@ -102,7 +371,8 @@ async def test_options():
             'title': 'Blue',
             'payload': 0x0000ff,
         }],
-        user=user)
+        user=user,
+    )
 
     mock_http.post.assert_called_with(
         'https://graph.facebook.com/v2.6/me/messages/',
@@ -360,14 +630,14 @@ async def test_handler_raw_text(build_fb_interface):
     @story.on('hello, world!')
     def correct_story():
         @story.part()
-        def store_result(message):
-            correct_trigger.receive(message)
+        def store_result(ctx):
+            correct_trigger.receive(story_context.get_message_data(ctx))
 
     @story.on('Goodbye, world!')
     def incorrect_story():
         @story.part()
-        def store_result(message):
-            incorrect_trigger.receive(message)
+        def store_result(ctx):
+            incorrect_trigger.receive(story_context.get_message_data(ctx))
 
     await fb_interface.handle({
         'object': 'page',
@@ -395,12 +665,8 @@ async def test_handler_raw_text(build_fb_interface):
 
     assert incorrect_trigger.value is None
     assert correct_trigger.value == {
-        'user': fb_interface.storage.user,
-        'session': fb_interface.storage.session,
-        'data': {
-            'text': {
-                'raw': 'hello, world!'
-            }
+        'text': {
+            'raw': 'hello, world!'
         }
     }
 
@@ -415,14 +681,14 @@ async def test_handler_selected_option(build_fb_interface):
     @story.on(receive=option.Match('GREEN'))
     def correct_story():
         @story.part()
-        def store_result(message):
-            correct_trigger.receive(message)
+        def store_result(ctx):
+            correct_trigger.receive(story_context.get_message_data(ctx))
 
     @story.on(receive=option.Match('BLUE'))
     def incorrect_story():
         @story.part()
-        def store_result(message):
-            incorrect_trigger.receive(message)
+        def store_result(ctx):
+            incorrect_trigger.receive(story_context.get_message_data(ctx))
 
     await fb_interface.handle({
         'object': 'page',
@@ -451,13 +717,9 @@ async def test_handler_selected_option(build_fb_interface):
 
     assert incorrect_trigger.value is None
     assert correct_trigger.value == {
-        'user': fb_interface.storage.user,
-        'session': fb_interface.storage.session,
-        'data': {
-            'option': 'GREEN',
-            'text': {
-                'raw': 'Green!'
-            }
+        'option': 'GREEN',
+        'text': {
+            'raw': 'Green!'
         }
     }
 
@@ -472,14 +734,14 @@ async def test_handler_postback(build_fb_interface):
     @story.on(receive=option.Match('GREEN'))
     def correct_story():
         @story.part()
-        def store_result(message):
-            correct_trigger.receive(message)
+        def store_result(ctx):
+            correct_trigger.receive(story_context.get_message_data(ctx))
 
     @story.on(receive=option.Match('BLUE'))
     def incorrect_story():
         @story.part()
-        def store_result(message):
-            incorrect_trigger.receive(message)
+        def store_result(ctx):
+            incorrect_trigger.receive(story_context.get_message_data(ctx))
 
     await fb_interface.handle({
         'object': 'page',
@@ -503,14 +765,45 @@ async def test_handler_postback(build_fb_interface):
 
     assert incorrect_trigger.value is None
     assert correct_trigger.value == {
-        'user': fb_interface.storage.user,
-        'session': fb_interface.storage.session,
-        'data': {
-            'option': 'GREEN',
-        }
+        'option': 'GREEN',
     }
 
 
+@pytest.mark.asyncio
+async def test_handler_thumbsup(build_fb_interface):
+    fb_interface, story = await build_fb_interface()
+
+    like_is_here_trigger = utils.SimpleTrigger()
+
+    @story.on(receive=sticker.Like())
+    def like_story():
+        @story.part()
+        def store_result(ctx):
+            like_is_here_trigger.passed()
+
+    await fb_interface.handle({
+        'object': 'page',
+        'entry': [{
+            'id': 'PAGE_ID',
+            'time': 1473204787206,
+            'messaging': [{
+                'sender': {
+                    'id': 'USER_ID'
+                },
+                'recipient': {
+                    'id': 'PAGE_ID'
+                },
+                'timestamp': 1458692752478,
+                'message': {
+                    'sticker_id': sticker.SMALL_LIKE,
+                }
+            }]
+        }]
+    })
+
+    assert like_is_here_trigger.is_passed()
+
+
 @pytest.mark.asyncio
 async def test_should_not_process_echo_delivery_and_read_messages_as_regular(build_fb_interface):
     fb_interface, story = await build_fb_interface()
