@@ -23,7 +23,7 @@ def build_deserialize(cls):
 
 def get_validator(filter_data):
     """
-    ask every matcher whether it can serve such filter ata
+    ask every matcher whether it can serve such filter data
 
     :param filter_data:
     :return:
@@ -57,6 +57,4 @@ def serialize(m):
 
 def deserialize(data):
     matcher_type = matchers[data['type']]
-    print('matcher_type')
-    print(help(matcher_type.deserialize))
     return matcher_type.deserialize(data['data'])
