@@ -1,6 +1,11 @@
+from botstory.ast import story_context
 from ... import matchers
 
 
+def get_option(ctx):
+    return story_context.get_message_data(ctx, 'option')
+
+
 @matchers.matcher()
 class Any:
     type = 'Option.Any'
@@ -8,8 +13,8 @@ class Any:
     def __init__(self):
         pass
 
-    def validate(self, message):
-        return message.get('data', {}).get('option', False)
+    def validate(self, ctx):
+        return get_option(ctx)
 
 
 @matchers.matcher()
@@ -19,8 +24,8 @@ class Match:
     def __init__(self, option):
         self.option = option
 
-    def validate(self, message):
-        return message.get('data', {}).get('option', None) == self.option
+    def validate(self, ctx):
+        return get_option(ctx) == self.option
 
     def serialize(self):
         return self.option
@@ -35,5 +40,5 @@ class OnStart:
     type = 'Option.OnStart'
     DEFAULT_OPTION_PAYLOAD = 'BOT_STORY.PUSH_GET_STARTED_BUTTON'
 
-    def validate(self, message):
-        return message.get('data', {}).get('option', None) == self.DEFAULT_OPTION_PAYLOAD
+    def validate(self, ctx):
+        return get_option(ctx) == self.DEFAULT_OPTION_PAYLOAD
